package com.kakao.sdk.common.model

import android.os.Parcelable
import com.google.gson.annotations.SerializedName
import com.kakao.sdk.common.json.UnknownValue
import kotlinx.android.parcel.Parcelize
import java.lang.RuntimeException

/**
 * @author kevin.kang. Created on 2019-06-04..
 */
@Suppress("Unused")
sealed class KakaoSdkError(open val msg: String) : RuntimeException(msg)

@Parcelize
data class ApiError(
    val statusCode: Int,
    val reason: ApiErrorCause,
    val response: ApiErrorResponse
) : KakaoSdkError(response.msg), Parcelable

@Parcelize
data class OAuthError(
    val statusCode: Int,
    val reason: AuthErrorCause,
    val response: AuthErrorResponse
) : KakaoSdkError(response.errorDescription), Parcelable

@Parcelize
data class ClientError(
    val reason: ClientErrorCause,
    override val msg: String = "Client-side error"
) :
    KakaoSdkError(msg), Parcelable

enum class AuthErrorCause {
    @SerializedName("invalid_request")
    InvalidRequest,
    @SerializedName("invalid_scope")
    InvalidScope,
    @SerializedName("invalid_grant")
    InvalidGrant,
    @SerializedName("misconfigured")
    Misconfigured,
    @SerializedName("unauthorized")
    Unauthorized,
    @SerializedName("access_denied")
    AccessDenied,
    @SerializedName("server_error")
    ServerError,
    @SerializedName("auto_login")
    AutoLogin,
    @UnknownValue
    Unknown,
}

enum class ApiErrorCause(val errorCode: Int) {
    @SerializedName("-1")
    InternalError(-1),
    @SerializedName("-2")
    IllegalParams(-2),
    @SerializedName("-3")
    UnsupportedApi(-3),
    @SerializedName("-4")
    BlockedAction(-4),
    @SerializedName("-5")
    PermissionDenied(-5),
    @SerializedName("-10")
    ApiLimitExceeded(-10),
    @SerializedName("-301")
    AppDoesNotExist(-301),
    @SerializedName("-401")
    InvalidToken(-401),
    @SerializedName("-403")
    InvalidOrigin(-403),
    @SerializedName("-603")
    TimeOut(-603),
    @SerializedName("-903")
    DeveloperDoesNotExist(-903),

    // authorized API
    @SerializedName("-101")
    NotRegisteredUser(-101),
    @SerializedName("-102")
    AlreadyRegisteredUser(-102),
    @SerializedName("-103")
    AccountDoesNotExist(-103),
    @SerializedName("-402")
    InvalidScope(-402),
    @SerializedName("-405")
    AgeVerificationRequired(-405),
    @SerializedName("-406")
    UnderAgeLimit(-406),

    // user API
    @SerializedName("-201")
    PropertyKeyDoesNotExist(-201),

    // talk API
    @SerializedName("-501")
    NotTalkUser(-501),
    @SerializedName("-502")
    NotFriend(-502),
    @SerializedName("-504")
    UserDeviceUnsupported(-504),
    @SerializedName("-530")
    TalkMessageDisabled(-530),
    @SerializedName("-531")
    TalkSendMessageMonthlyLimitExceed(-531),
    @SerializedName("-532")
    TalkSendMessageDailyLimitExceed(-532),

    @SerializedName("-601")
    NotStoryUser(-601),
    @SerializedName("-602")
    StoryImageUploadSizeExceeded(-602),
    @SerializedName("-604")
    StoryInvalidScrapUrl(-604),
    @SerializedName("-605")
    StoryInvalidPostId(-605),
    @SerializedName("-606")
    StoryMaxUploadCountExceed(-606),

    @UnknownValue
    Unknown(Int.MAX_VALUE);
}

enum class ClientErrorCause {
    Unknown,
    Cancelled,
    TokenNotFound,
    NetworkError,
    NotSupported,
    BadParameter,
    IllegalState
}


